/*
 * lexer.c: lexical analyser for datadec input language
 *          initially only the "inner" language, without print hints.
 *
 ******* Tokens:
 *
 *	tEQ, tOR, tSEMI, tOPENBR, tCOMMA, tCLOSEBR,
 *	tID, tERROR, tEOF,
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "bool.h"
#include "lexer.h"

/* ----------------- Exported variables ---------------- */

#define MAXIDSIZE 100
char lexidval[ MAXIDSIZE ];
int  lineno = 1;
int lexintval;
FILE *lexfile;

char *tokenname[] =
{
	"eq", "or", "semi", "openbr", "comma",
	"closebr", "id", "ERROR", "EOF",
	NULL
};

/* ----------------- Private procedures ---------------- */

static void skipwhitespace( void )
{
        int c;
        for(;;) {
                c = getc(lexfile);
		if( c == '/' || c == '#' ) /* comment to end of line */
		{
			while( (c=getc(lexfile)) != EOF && c != '\n' );
		}
                if( c == EOF ) break;
                if( c == '\n' ) lineno++;
                if( c != ' ' && c != '\t' && c != '\n' ) break;
        }
        ungetc( c, lexfile );
}

/* ----------------- Private variables  ---------------- */

static BOOL  havepushedtok = FALSE;
static TOKEN curtok;

/* ----------------- Public procedures  ---------------- */

void ungettok( void )
{
        if( havepushedtok )
	{
		fprintf( stderr, "ungettok: can't push 2 tokens\n" );
		exit(1);
	}
        havepushedtok = TRUE;
#ifdef DEBUGGING
printf( "lexer: ungot token %s\n", tokenname[ curtok ] );
#endif
}

TOKEN nexttok( void )
{
	int c;
	int  pos;
        if( havepushedtok )
	{
                havepushedtok = FALSE;
#ifdef DEBUGGING
printf( "returning token %s\n", tokenname[ curtok ] );
#endif
		return curtok;
        }
	skipwhitespace();
	c = getc(lexfile);
	switch( c )
	{
	case EOF: curtok = tEOF; break;
	case ';': curtok = tSEMI; break;
	case '(': curtok = tOPENBR; break;
	case ')': curtok = tCLOSEBR; break;
	case ',': curtok = tCOMMA; break;
	case '|': curtok = tOR; break;
	case '=': curtok = tEQ; break;
	default:
		curtok = tERROR;
		if( isalpha(c) )
		{
			curtok = tID;
			for( pos=0; isalpha(c)||isdigit(c)||c=='_'; )
			{
				if( pos<MAXIDSIZE-1 )
					lexidval[pos++] = c;
				c = getc(lexfile);
			}
			ungetc( c, lexfile );
			while( pos<MAXIDSIZE )
			{
				lexidval[pos++] = '\0';
			}
			if( streq( lexidval, "OR" ) || streq( lexidval, "or" ) )
			{
				curtok = tOR;
			}
		}
	}
#ifdef DEBUGGING
printf( "returning token %s\n", tokenname[ curtok ] );
#endif
        return curtok;
}
